@@ -203,6 +203,7 @@ class PrestoRequest(object):
         catalog=None,  # type: Text
         schema=None,  # type: Text
         session_properties=None,  # type: Optional[Dict[Text, Any]]
+        http_session=None,  # type: Any
         http_headers=None,  # type: Optional[Dict[Text, Text]]
         transaction_id=NO_TRANSACTION,  # type: Optional[Text]
         http_scheme=constants.HTTP,  # type: Text
@@ -226,10 +227,13 @@ class PrestoRequest(object):
         self._host = host
         self._port = port
         self._next_uri = None  # type: Optional[Text]
-        # mypy cannot follow module import
-        self._http_session = self.http.Session()  # type: ignore
+
+        if http_session is not None:
+            self._http_session = http_session
+        else:
+            # mypy cannot follow module import
+            self._http_session = self.http.Session()  # type: ignore
         self._http_session.headers.update(self.http_headers)
-        self._http_cookies = {}
         self._auth = auth
         if self._auth:
             if http_scheme == constants.HTTP:
@@ -305,19 +309,9 @@ class PrestoRequest(object):
             ),
             max_attempts=self._max_attempts,
         )
-        def get(*args, **kwargs):
-            return self._http_session.get(
-                *args, cookies=self._http_cookies, **kwargs)
-        def post(*args, **kwargs):
-            return self._http_session.post(
-                *args, cookies=self._http_cookies, **kwargs)
-        def delete(*args, **kwargs):
-            return self._http_session.delete(
-                *args, cookies=self._http_cookies, **kwargs)
-
-        self._get = with_retry(get)
-        self._post = with_retry(post)
-        self._delete = with_retry(delete)
+        self._get = with_retry(self._http_session.get)
+        self._post = with_retry(self._http_session.post)
+        self._delete = with_retry(self._http_session.delete)
 
     def get_url(self, path):
         # type: (Text) -> Text
@@ -424,14 +418,11 @@ class PrestoRequest(object):
 
         if constants.HEADER_SET_SESSION in http_response.headers:
             for key, value in get_session_property_values(
-                    response.headers,
-                    constants.HEADER_SET_SESSION,
-                ):
+                response.headers,
+                constants.HEADER_SET_SESSION,
+            ):
                 self._client_session.properties[key] = value
 
-        if http_response.cookies:
-            self._http_cookies.update(http_response.cookies)
-
         self._next_uri = response.get('nextUri')
 
         return PrestoStatus(
